@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2006-2016 Julien Pommier.
+ Copyright (C) 2006-2020 Julien Pommier.
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -18,7 +18,7 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_mesh_fem_get.cc 5353 2016-07-26 13:36:19Z renard $
+// $Id$
 
 #include <getfem/getfem_fem.h>
 #include <getfem/getfem_export.h>
@@ -121,7 +121,7 @@ non_conformal_dof(getfem::mesh_fem &mf, mexargs_in &in, mexargs_out &out) {
 
     for (short_type f = 0; f < m.structure_of_convex(ic)->nb_faces(); f++) {
       bgeot::short_type q;
-      if (!m.is_convex_having_neighbour(ic, f)) {
+      if (!m.is_convex_having_neighbor(ic, f)) {
         q = 2;
       } else {
         q = 1;
@@ -136,7 +136,7 @@ non_conformal_dof(getfem::mesh_fem &mf, mexargs_in &in, mexargs_out &out) {
   iarray w = out.pop().create_iarray_h(
     unsigned(std::count_if(dcnt.begin(),
                            dcnt.end(),
-                           std::bind2nd(std::equal_to<bgeot::short_type>(),1))));
+                           [](const auto &x) {return x == 1;})));
   size_type i,j=0;
   /*
   std::copy_if(dcnt.begin(), dcnt.end(),
@@ -434,7 +434,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET CVs = ('convex_index')
-    Return the list of convexes who have a FEM.@*/
+    Return the list of convexes who have an FEM.@*/
     sub_command
       ("convex_index", 0, 0, 0, 1,
        out.pop().from_bit_vector(mf->convex_index());
@@ -509,40 +509,39 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
        );
 
     /*@GET Vr = ('reduce vector', @vec V)
-    Multiply the provided vector V with the extension matrix of the @tmf.@*/
+      Multiply the provided vector V with the extension matrix of the @tmf.@*/
     sub_command
       ("reduce vector", 1, 1, 0, 1,
        if (in.front().is_complex()) {
-         carray V = in.pop().to_carray(-1);
-         std::vector<std::complex<double> > Vr(mf->nb_dof());
-         mf->reduce_vector(V, Vr);
-         out.pop().from_dcvector(Vr);
+	 carray V = in.pop().to_carray(-1);
+	 std::vector<std::complex<double> > Vr(mf->nb_dof());
+	 mf->reduce_vector(V, Vr);
+	 out.pop().from_dcvector(Vr);
        } else {
-         darray V = in.pop().to_darray(-1);
-         std::vector<double> Vr(mf->nb_dof());
-         mf->reduce_vector(V, Vr);
-         out.pop().from_dcvector(Vr);
+	 darray V = in.pop().to_darray(-1);
+	 std::vector<double> Vr(mf->nb_dof());
+	 mf->reduce_vector(V, Vr);
+	 out.pop().from_dcvector(Vr);
        }
        );
-
+    
     /*@GET Ve = ('extend vector', @vec V)
-    Multiply the provided vector V with the reduction matrix of the @tmf.@*/
+      Multiply the provided vector V with the reduction matrix of the @tmf.@*/
     sub_command
       ("extend vector", 1, 1, 0, 1,
        if (in.front().is_complex()) {
-         carray V = in.pop().to_carray(-1);
-         std::vector<std::complex<double> > Ve(mf->nb_basic_dof());
-         mf->extend_vector(V, Ve);
-         out.pop().from_dcvector(Ve);
+	 carray V = in.pop().to_carray(-1);
+	 std::vector<std::complex<double> > Ve(mf->nb_basic_dof());
+	 mf->extend_vector(V, Ve);
+	 out.pop().from_dcvector(Ve);
        } else {
-         darray V = in.pop().to_darray(-1);
-         std::vector<double> Ve(mf->nb_basic_dof());
-         mf->extend_vector(V, Ve);
-         out.pop().from_dcvector(Ve);
+	 darray V = in.pop().to_darray(-1);
+	 std::vector<double> Ve(mf->nb_basic_dof());
+	 mf->extend_vector(V, Ve);
+	 out.pop().from_dcvector(Ve);
        }
        );
-
-
+    
     /*@GET DOFs = ('basic dof on region',@mat Rs)
     Return the list of basic dof (before the optional reduction) lying on one
     of the mesh regions listed in `Rs`.
@@ -577,7 +576,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
     For a reduced mesh_fem
     a dof is lying on a region if its potential corresponding shape
     function is nonzero on this region. The extension matrix is used
-    to make the correspondance between basic and reduced dofs.@*/
+    to make the correspondence between basic and reduced dofs.@*/
     sub_command
       ("dof on region", 1, 1, 0, 1,
        iarray bnums = in.pop().to_iarray(-1);
@@ -642,7 +641,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
     Return the array which associates an integer (the partition number)
     to each convex of the @tmf. By default, it is an all-zero array.
     The degrees of freedom of each convex of the @tmf are connected
-    only to the dof of neighbouring convexes which have the same
+    only to the dof of neighboring convexes which have the same
     partition number, hence it is possible to create partially
     discontinuous @tmf very easily.@*/
     sub_command
@@ -655,7 +654,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
 
 
     /*@GET ('save',@str filename[, @str opt])
-    Save a @tmf in a text file (and optionaly its linked mesh object
+    Save a @tmf in a text file (and optionally its linked mesh object
     if `opt` is the string 'with_mesh').@*/
     sub_command
       ("save", 1, 2, 0, 0,
@@ -869,7 +868,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
 
     Interpolate data given on each convex of the mesh to the @tmf dof.
     The @tmf has to be lagrangian, and should be discontinuous (typically
-    a FEM_PK(N,0) or FEM_QK(N,0) should be used).
+    an FEM_PK(N,0) or FEM_QK(N,0) should be used).
 
     The last dimension of the input vector Ucv should have
     MESH:GET('max cvid') elements.
@@ -971,18 +970,18 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
       nbd = P.shape[1]
     vars = ('x','y','z','u','v','w')
     nbvars = min(P.shape[0],len(vars))
-    for i in xrange(0,nbvars):
+    for i in range(0,nbvars):
       gl[vars[i]] = P[i,0]
       lo[vars[i]] = P[i,0]
     ccode = compile(expression, '<string>', 'eval');
     r = numpy.array(eval(ccode,gl,lo))
     Z = numpy.zeros(r.shape + (nbd,), r.dtype)
-    nbd_p = nbd/nbp
+    nbd_p = int(nbd/nbp)
     nbd_end = nbd_p*(rk+1)
     if (rk == nbp-1):
       nbd_end = nbd
-    for j in xrange(nbd_p*rk,nbd_end):
-      for i in xrange(0,nbvars):
+    for j in range(nbd_p*rk,nbd_end):
+      for i in range(0,nbvars):
         gl[vars[i]] = P[i,j]
         lo[vars[i]] = P[i,j]
       Z[...,j] = eval(ccode,gl,lo)
@@ -1009,7 +1008,7 @@ void gf_mesh_fem_get(getfemint::mexargs_in& m_in,
 
   @*/
 /*@MATLABEXT
-  if (nargin>=2 & strcmpi(varargin{2},'eval')),
+  if (nargin>=2 && strcmpi(varargin{2},'eval')),
     [varargout{1:nargout}]=gf_mesh_fem_get_eval(varargin{[1 3:nargin]}); return;
   end;
   @*/
