@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2006-2016 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2020 Yves Renard, Julien Pommier.
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -18,7 +18,7 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_slice_get.cc 5353 2016-07-26 13:36:19Z renard $
+// $Id$
 #include <map>
 #include <getfemint_workspace.h>
 #include <getfem/getfem_mesh_slice.h>
