@@ -1,10 +1,10 @@
 /*===========================================================================
 
- Copyright (C) 2001-2015 Yves Renard
+ Copyright (C) 2001-2020 Yves Renard
 
- This file is a part of GetFEM++
+ This file is a part of GetFEM
 
- GetFEM++  is  free software;  you  can  redistribute  it  and/or modify it
+ GetFEM  is  free software;  you  can  redistribute  it  and/or modify it
  under  the  terms  of the  GNU  Lesser General Public License as published
  by  the  Free Software Foundation;  either version 3 of the License,  or
  (at your option) any later version along with the GCC Runtime Library
@@ -56,7 +56,7 @@ namespace getfem {
     bool all_convexes = (rg_source.id() == mesh_region::all_convexes().id());
 
     size_type nbpts = nb_points();
-    size_type nbcvx = msh.convex_index().last_true() + 1;
+    size_type nbcvx = msh.nb_allocated_convex();
     ref_coords.resize(nbpts);
     std::vector<double> dist(nbpts);
     std::vector<size_type> cvx_pts(nbpts);
@@ -67,6 +67,8 @@ namespace getfem {
     npt.add(0, nbpts);
     scalar_type mult = scalar_type(1);
 
+    bool projection_into_element(extrapolation == 0);
+
     do {
       for (dal::bv_visitor j(rg_source.index()); !j.finished(); ++j) {
         if (mult > scalar_type(1) && !(cv_on_bound.is_in(j))) continue;
@@ -76,10 +78,10 @@ namespace getfem {
         if (extrapolation == 2) {
           if (mult == scalar_type(1))
             for (short_type f = 0; f < msh.nb_faces_of_convex(j); ++f) {
-              size_type neighbour_cv = msh.neighbour_of_convex(j, f);
-              if (!all_convexes && neighbour_cv != size_type(-1)) {
-                // check if the neighbour is also contained in rg_source ...
-                if (!rg_source.is_in(neighbour_cv)) 
+              size_type neighbor_cv = msh.neighbor_of_convex(j, f);
+              if (!all_convexes && neighbor_cv != size_type(-1)) {
+                // check if the neighbor is also contained in rg_source ...
+                if (!rg_source.is_in(neighbor_cv)) 
                   cv_on_bound.add(j); // ... if not, treat the element as a boundary one
               }
               else // boundary element of the overall mesh
@@ -101,7 +103,8 @@ namespace getfem {
           size_type ind = boxpts[l].i;
           if (npt[ind] || dist[ind] > 0) {
             bool converged;
-            bool gicisin = gic.invert(boxpts[l].n, pt_ref, converged, EPS);
+            bool gicisin = gic.invert(boxpts[l].n, pt_ref, converged, EPS,
+                                      projection_into_element);
             bool toadd = extrapolation || gicisin;
             double isin = pgt->convex_ref()->is_in(pt_ref);
             
